var request = require('request');

function KahootAPI() {
    if(!(this instanceof KahootAPI)) {
        return new KahootAPI();
    }

    this.session = {
        token: "",
        token_expiry: 0,
        user: {
            username: "",
            email: "",
            uuid: ""
        }
    };


    /**
     * rawRequest - make a request to kahoot
     *
     * @param  {object} data   contains all information
     * @return {Promise}
     */
    this.rawRequest = function(data) {
        if(typeof data.method === 'undefined') data.method = 'GET';

        return new Promise((resolve, reject) => {
            request({
                uri: "https://create.kahoot.it/rest/" + data.uri,
                method: data.method,
                json: data.fields,
                headers: {
                    'Content-Type': 'application/json'
                }
            }, (error, response, body) => {
                if(error) {
                    reject(Error(
                        error.stringify()
                    ));
                } else {
                    if(typeof body.error === 'undefined') {
                        resolve(body);
                    } else {
                        reject(Error(
                            'Received error from Kahoot! API endpoint: ' +
                            body.error + ' (' +
                            body.error_description + ')'
                        ));
                    }
                }
            });
        });
    };

    /**
     * request - make a request to kahoot
     *
     * @param  {object} data   contains all information
     * @return {Promise}
     */
    this.request = function(data) {
        if(this.session.token.length === 0 || this.session.token_expiry === 0)
            return Error(
                'This request requires a session token'
            );

        if(this.session.token_expiry < Math.floor(new Date() / 1000))
            return Error(
                'The auth token has expired!'
            );

        if(typeof data.method === 'undefined') data.method = 'GET';

        return new Promise((resolve, reject) => {
            request({
                uri: "https://create.kahoot.it/rest/" + data.uri,
                method: data.method,
                json: data.fields,
                headers: {
                    'Content-Type': 'application/json',
                    'Authorization': 'Bearer ' + this.session.token
                }
            }, (error, response, body) => {
                if(error) {
                    reject(Error(
                        error.stringify()
                    ));
                } else {
                    if(typeof body.error === 'undefined') {
                        resolve(body);
                    } else {
                        reject(Error(
                            'Received error from Kahoot! API endpoint: ' +
                            body.error + ' (' +
                            body.error_description + ')'
                        ));
                    }
                }
            });
        });
    };

    /**
     * login - logins with an username (email) and password with
     * Kahoot! REST API
     *
     * @param  {string} username user's username
     * @param  {string} password user's password
     * @return {Promise}
     */
    this.login = function(username, password) {
        return this.rawRequest({
            uri: 'authenticate',
            method: 'POST',
            fields: {
                "username": username,
                "password": password,
                "grant_type": "password"
            }
        }).then((data) => {
            this.session.token = data.access_token;
            this.session.token_expiry = data.expires;
            this.session.user.username = data.user.username;
            this.session.user.email = data.user.email;
            this.session.user.uuid = data.user.uuid;

            return this.session;
        });
    };


    /**
     * search - searches public Kahoots
     *
     * @param  {string} query       search query
     * @param  {number} [limit=30]  set the limit of search results
     * @return {Promise}
     */
    this.search = function(query, limit) {
        if(typeof limit === 'undefined') limit = 30;

        return this.request({
            uri: 'kahoots/search/public?' +
                'query=' + encodeURIComponent(query) + '&limit=' + limit,
            method: 'GET',
            fields: null
        }).then((data) => {
            return JSON.parse(data).entities;
        });
    };
}

module.exports = {
    API: KahootAPI
};
